package com.mortals.xhx.common.utils;

import org.apache.poi.hssf.usermodel.*;
import org.apache.poi.ss.formula.functions.T;
import org.apache.poi.ss.usermodel.PictureData;
import org.springframework.web.multipart.MultipartFile;

import java.io.*;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.stream.Collectors;


public class ReadExcelPictureUtil {

    /**
     * 每个单元格单张图片
     * @param uploadFile
     * @return
     * @throws IOException
     */
    public static Map<String, PictureData> readExcelPictureMap( MultipartFile uploadFile) throws IOException {
        //获取所有图片
        HSSFWorkbook hssfWorkbook = new HSSFWorkbook(uploadFile.getInputStream());// 工作簿

        HSSFSheet sheet = hssfWorkbook.getSheetAt(0);// 工作表

//        int lastRowNum = sheet.getLastRowNum();// 获取最后一行序号,从零开始

        Map<String, PictureData> picMap = new HashMap<>();// 存储图片信息和坐标

//        List<HSSFShape> list = Optional.ofNullable(sheet.getDrawingPatriarch().getChildren()).orElse(null);
        List<HSSFShape> list = Optional.ofNullable(sheet).map(HSSFSheet::getDrawingPatriarch).map(HSSFPatriarch::getChildren).orElse(null);

        if (list != null && list.size() > 0) {// 处理获取图片信息和坐标
            list = list.stream().filter(item -> item instanceof HSSFPicture).collect(Collectors.toList());// 过滤出图片的数据
            for (HSSFShape hssfShape : list) {
                HSSFPicture hSSFPicture = (HSSFPicture) hssfShape;
                HSSFClientAnchor hSSFClientAnchor = (HSSFClientAnchor) hSSFPicture.getAnchor();
                PictureData pictureData = hSSFPicture.getPictureData();
                if(pictureData!=null) {
                    String point = hSSFClientAnchor.getRow1() + "," + hSSFClientAnchor.getCol1();
                    picMap.put(point, pictureData);
                }
            }
        }
        return picMap;//图片map集合
    }

    //将上面所有的图片map集合循环，每次循环调用这个方法，进行每条数据的多张图片进行保存，返回图片的路径集合
    public static Map<String, String> savePicture(String root, Map<String, PictureData> picMap){
        Map<String,String> fileMap = new HashMap<>();
        Iterator<Map.Entry<String, PictureData>> it = picMap.entrySet().iterator();
        while (it.hasNext()) {
            Map.Entry<String, PictureData> entry = it.next();
            //System.out.println("key= " + entry.getKey() + " and value= " + entry.getValue());
            PictureData pictureData = entry.getValue();
            File picDirF = null;

            String imageName = new Date().getTime() + ".png";
            String imagePath = "/file/uploadfile/"+imageName;
            String filePath = root + imagePath;

            FileOutputStream fout;
            try {
                fout = new FileOutputStream(filePath);
                fout.write(pictureData.getData());
                fout.close();
            } catch (Exception e) {
                e.printStackTrace();
            }
            fileMap.put(entry.getKey(),imagePath);
        }
        return fileMap;//返回图片路径集合
    }

    public static void main(String[] args) {
        String filepath = "D:\\mortals\\app\\data\\cpm\\excel\\test3.xls";
        try {
            InputStream is = new FileInputStream(filepath);
            //获取所有图片
            HSSFWorkbook hssfWorkbook = new HSSFWorkbook(is);// 工作簿

            HSSFSheet sheet = hssfWorkbook.getSheetAt(0);// 工作表

//        int lastRowNum = sheet.getLastRowNum();// 获取最后一行序号,从零开始

            Map<String, String> picMap = new HashMap<>();// 存储图片信息和坐标

//        List<HSSFShape> list = Optional.ofNullable(sheet.getDrawingPatriarch().getChildren()).orElse(null);
            List<HSSFShape> list = Optional.ofNullable(sheet).map(HSSFSheet::getDrawingPatriarch).map(HSSFPatriarch::getChildren).orElse(null);

            if (list != null && list.size() > 0) {// 处理获取图片信息和坐标
                list = list.stream().filter(item -> item instanceof HSSFPicture).collect(Collectors.toList());// 过滤出图片的数据
                for (HSSFShape hssfShape : list) {
                    HSSFPicture hSSFPicture = (HSSFPicture) hssfShape;
                    HSSFClientAnchor hSSFClientAnchor = (HSSFClientAnchor) hSSFPicture.getAnchor();
                    PictureData pictureData = hSSFPicture.getPictureData();
                    if (pictureData != null) {
                        String point = hSSFClientAnchor.getRow1() + "," + hSSFClientAnchor.getCol1();

                        String imageName = new Date().getTime() + ".png";
                        String imagePath = "/file/uploadfile/"+imageName;
                        String filePath = "D:/mortals/app/data/cpm" + imagePath;

                        FileOutputStream fout;
                        try {
                            fout = new FileOutputStream(filePath);
                            fout.write(pictureData.getData());
                            fout.close();
                        } catch (Exception e) {
                            e.printStackTrace();
                        }
                        System.out.println(imagePath);
                        picMap.put(point, imagePath);
                    }
                }
            }
            System.out.println(picMap);
        }catch (Exception e){

        }

    }
}
